@@ -1,12 +1,12 @@
 #!/bin/sh
 set -e
 version=`./setup.py --version`
+rm -rf build dist
 ./setup.py compile_catalog
 ./setup-freeze.py bdist_mac
-rm -rf dist
 mkdir dist
 mv build/Tryton.app dist/
-for f in CHANGELOG COPYRIGHT LICENSE README; do
+for f in CHANGELOG COPYRIGHT LICENSE; do
     cp ${f} dist/${f}.txt
 done
 cp -r doc dist/
