@@ -11,9 +11,7 @@ import traceback
 import webbrowser
 from urllib.parse import urlparse, parse_qsl, unquote
 
-import gobject
-import gtk
-from gi.repository import Gtk, GLib, Gio
+from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
 
 import tryton.common as common
 import tryton.plugins
@@ -50,17 +48,18 @@ class Main(Gtk.Application):
         action = Gio.SimpleAction.new('preferences', None)
         action.connect('activate', lambda *a: self.preferences())
         self.add_action(action)
+        self.set_accels_for_action('app.preferences', ['<Primary>comma'])
 
         action = Gio.SimpleAction.new('menu-search', None)
         action.connect(
             'activate', lambda *a: self.global_search_entry.grab_focus())
         self.add_action(action)
-        self.add_accelerator('<Primary>k', 'app.menu-search')
+        self.set_accels_for_action('app.menu-search', ['<Primary>k'])
 
         action = Gio.SimpleAction.new('menu-toggle', None)
         action.connect('activate', lambda *a: self.menu_toggle())
         self.add_action(action)
-        self.add_accelerator('<Primary>m', 'app.menu-toggle')
+        self.set_accels_for_action('app.menu-toggle', ['<Primary>m'])
 
         toolbar_variant = GLib.Variant.new_string(
             CONFIG['client.toolbar'] or 'both')
@@ -77,9 +76,8 @@ class Main(Gtk.Application):
 
         for name, key in [
                 ('mode-pda', 'client.modepda'),
-                ('save-width-height', 'client.save_width_height'),
+                ('save-tree-width', 'client.save_tree_width'),
                 ('save-tree-state', 'client.save_tree_state'),
-                ('fast-tabbing', 'client.fast_tabbing'),
                 ('spell-checking', 'client.spellcheck'),
                 ('check-version', 'client.check_version'),
                 ]:
@@ -91,24 +89,22 @@ class Main(Gtk.Application):
         action = Gio.SimpleAction.new('tab-previous', None)
         action.connect('activate', lambda *a: self.win_prev())
         self.add_action(action)
-        self.add_accelerator('<Primary>Left', 'app.tab-previous')
+        self.set_accels_for_action('app.tab-previous', ['<Primary><Shift>Tab'])
 
         action = Gio.SimpleAction.new('tab-next', None)
         action.connect('activate', lambda *a: self.win_next())
         self.add_action(action)
-        self.add_accelerator('<Primary>Right', 'app.tab-next')
+        self.set_accels_for_action('app.tab-next', ['<Primary>Tab'])
 
         action = Gio.SimpleAction.new('search-limit', None)
         action.connect('activate', lambda *a: self.edit_limit())
         self.add_action(action)
 
-        action = Gio.SimpleAction.new('email', None)
-        action.connect('activate', lambda *a: self.edit_email())
-        self.add_action(action)
-
+        self._shortcuts = None
         action = Gio.SimpleAction.new('shortcuts', None)
         action.connect('activate', lambda *a: self.shortcuts())
         self.add_action(action)
+        self.set_accels_for_action('app.shortcuts', ['<Primary>F1'])
 
         action = Gio.SimpleAction.new('about', None)
         action.connect('activate', lambda *a: self.about())
@@ -117,43 +113,7 @@ class Main(Gtk.Application):
         action = Gio.SimpleAction.new('quit', None)
         action.connect('activate', self.on_quit)
         self.add_action(action)
-        self.add_accelerator('<Primary>q', 'app.quit')
-
-        menu = Gio.Menu.new()
-        menu.append(_("Preferences..."), 'app.preferences')
-
-        section = Gio.Menu.new()
-        toolbar = Gio.Menu.new()
-        section.append_submenu(_("Toolbar"), toolbar)
-        toolbar.append(_("Default"), 'app.toolbar::default')
-        toolbar.append(_("Text and Icons"), 'app.toolbar::both')
-        toolbar.append(_("Text"), 'app.toolbar::text')
-        toolbar.append(_("Icons"), 'app.toolbar::icons')
-
-        form = Gio.Menu.new()
-        section.append_submenu(_("Form"), form)
-        form.append(_("Save Width/Height"), 'app.save-width-height')
-        form.append(_("Save Tree State"), 'app.save-tree-state')
-        form.append(_("Fast Tabbing"), 'app.fast-tabbing')
-        form.append(_("Spell Checking"), 'app.spell-checking')
-
-        section.append(_("PDA Mode"), 'app.mode-pda')
-        section.append(_("Search Limit..."), 'app.search-limit')
-        section.append(_("Email..."), 'app.email')
-        section.append(_("Check Version"), 'app.check-version')
-
-        menu.append_section(_("Options"), section)
-
-        section = Gio.Menu.new()
-        section.append(_("Keyboard Shortcuts..."), 'app.shortcuts')
-        section.append(_("About..."), 'app.about')
-        menu.append_section(_("Help"), section)
-
-        section = Gio.Menu.new()
-        section.append(_("Quit"), 'app.quit')
-        menu.append_section(None, section)
-
-        self.set_app_menu(menu)
+        self.set_accels_for_action('app.quit', ['<Primary>q'])
 
     def do_activate(self):
         if self.window:
@@ -163,19 +123,24 @@ class Main(Gtk.Application):
         self.window = Gtk.ApplicationWindow(application=self, title="Tryton")
         self.window.set_default_size(960, 720)
         self.window.maximize()
-        self.window.set_position(Gtk.WIN_POS_CENTER)
+        self.window.set_position(Gtk.WindowPosition.CENTER)
         self.window.set_resizable(True)
         self.window.set_icon(TRYTON_ICON)
         self.window.connect("destroy", self.on_quit)
         self.window.connect("delete_event", self.on_quit)
+        common.setup_window(self.window)
 
         self.header = Gtk.HeaderBar.new()
         self.header.set_show_close_button(True)
         self.window.set_titlebar(self.header)
         self.set_title()
 
+        self.primary_menu = Gtk.MenuButton.new()
+        self.primary_menu.set_menu_model(self._get_primary_menu())
+        self.header.pack_end(self.primary_menu)
+
         menu = Gtk.Button.new()
-        menu .set_relief(Gtk.ReliefStyle.NONE)
+        menu.set_relief(Gtk.ReliefStyle.NONE)
         menu.set_image(
             common.IconFactory.get_image('tryton-menu', Gtk.IconSize.BUTTON))
         menu.connect('clicked', self.menu_toggle)
@@ -187,7 +152,7 @@ class Main(Gtk.Application):
                 'tryton-bookmarks', Gtk.IconSize.BUTTON))
         self.menu_favorite = Gtk.Menu.new()
         favorite.set_popup(self.menu_favorite)
-        favorite.connect('clicked', self.favorite_set)
+        favorite.connect('button-press-event', self.favorite_set)
         self.header.pack_start(favorite)
 
         self.set_global_search()
@@ -196,64 +161,73 @@ class Main(Gtk.Application):
         self.accel_group = Gtk.AccelGroup()
         self.window.add_accel_group(self.accel_group)
 
-        gtk.accel_map_add_entry('<tryton>/Form/New', gtk.keysyms.N,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Save', gtk.keysyms.S,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Duplicate', gtk.keysyms.D,
-                gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Delete', gtk.keysyms.D,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Next', gtk.keysyms.Page_Down,
-                0)
-        gtk.accel_map_add_entry('<tryton>/Form/Previous', gtk.keysyms.Page_Up,
-                0)
-        gtk.accel_map_add_entry('<tryton>/Form/Switch View', gtk.keysyms.L,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Close', gtk.keysyms.W,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Reload', gtk.keysyms.R,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Attachments', gtk.keysyms.T,
-            gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Notes', gtk.keysyms.O,
-            gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Relate', gtk.keysyms.R,
-            gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Actions', gtk.keysyms.E,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Report', gtk.keysyms.P,
-                gtk.gdk.CONTROL_MASK)
-        gtk.accel_map_add_entry('<tryton>/Form/Search', gtk.keysyms.F,
-            gtk.gdk.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/New', Gdk.KEY_N, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Save', Gdk.KEY_S, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Duplicate', Gdk.KEY_D,
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Delete', Gdk.KEY_D, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Next', Gdk.KEY_Page_Down, 0)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Previous', Gdk.KEY_Page_Up, 0)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Switch View', Gdk.KEY_L,
+            Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Close', Gdk.KEY_W, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Reload', Gdk.KEY_R, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Attachments', Gdk.KEY_T,
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Notes', Gdk.KEY_O,
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Relate', Gdk.KEY_R,
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Actions', Gdk.KEY_E, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Report', Gdk.KEY_P, Gdk.ModifierType.CONTROL_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Email', Gdk.KEY_E,
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
+        Gtk.AccelMap.add_entry(
+            '<tryton>/Form/Search', Gdk.KEY_F,
+            Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.SHIFT_MASK)
 
-        gtk.accel_map_load(os.path.join(get_config_dir(), 'accel.map'))
+        Gtk.AccelMap.load(os.path.join(get_config_dir(), 'accel.map'))
 
         self.tooltips = common.Tooltips()
 
-        self.vbox = gtk.VBox()
+        self.vbox = Gtk.VBox()
         self.window.add(self.vbox)
 
         self.buttons = {}
 
-        self.info = gtk.VBox()
-        self.vbox.pack_start(self.info, expand=False)
+        self.info = Gtk.VBox()
+        self.vbox.pack_start(self.info, expand=False, fill=True, padding=0)
         if CONFIG['client.check_version']:
             common.check_version(self.info)
             GLib.timeout_add_seconds(
                 int(CONFIG['download.frequency']), common.check_version,
                 self.info)
 
-        self.pane = gtk.HPaned()
-        self.vbox.pack_start(self.pane, True, True)
+        self.pane = Gtk.HPaned()
+        self.vbox.pack_start(self.pane, expand=True, fill=True, padding=0)
         self.pane.set_position(int(CONFIG['menu.pane']))
 
         self.menu_screen = None
-        self.menu = gtk.VBox()
+        self.menu = Gtk.VBox()
         self.menu.set_vexpand(True)
         self.pane.add1(self.menu)
 
-        self.notebook = gtk.Notebook()
+        self.notebook = Gtk.Notebook()
         self.notebook.popup_enable()
         self.notebook.set_scrollable(True)
         self.notebook.connect_after('switch-page', self._sig_page_changt)
@@ -276,7 +250,7 @@ class Main(Gtk.Application):
         except Exception as exception:
             if (not isinstance(exception, TrytonError)
                     or exception.faultCode != 'QueryCanceled'):
-                common.error(str(exception), traceback.format_exc())
+                common.error(exception, traceback.format_exc())
             return self.quit()
         self.get_preferences()
 
@@ -303,18 +277,52 @@ class Main(Gtk.Application):
         rpc.logout()
         self.quit()
 
+    def add_window(self, window):
+        super().add_window(window)
+        common.setup_window(window)
+
+    def _get_primary_menu(self):
+        menu = Gio.Menu.new()
+        menu.append(_("Preferences..."), 'app.preferences')
+
+        section = Gio.Menu.new()
+        toolbar = Gio.Menu.new()
+        section.append_submenu(_("Toolbar"), toolbar)
+        toolbar.append(_("Default"), 'app.toolbar::default')
+        toolbar.append(_("Text and Icons"), 'app.toolbar::both')
+        toolbar.append(_("Text"), 'app.toolbar::text')
+        toolbar.append(_("Icons"), 'app.toolbar::icons')
+
+        form = Gio.Menu.new()
+        section.append_submenu(_("Form"), form)
+        form.append(_("Save Column Width"), 'app.save-tree-width')
+        form.append(_("Save Tree State"), 'app.save-tree-state')
+        form.append(_("Spell Checking"), 'app.spell-checking')
+
+        section.append(_("PDA Mode"), 'app.mode-pda')
+        section.append(_("Search Limit..."), 'app.search-limit')
+        section.append(_("Check Version"), 'app.check-version')
+
+        menu.append_section(_("Options"), section)
+
+        section = Gio.Menu.new()
+        section.append(_("Keyboard Shortcuts..."), 'app.shortcuts')
+        section.append(_("About..."), 'app.about')
+        menu.append_section(_("Help"), section)
+        return menu
+
     def set_global_search(self):
         self.global_search_entry = Gtk.Entry.new()
         self.global_search_entry.set_width_chars(20)
-        global_search_completion = gtk.EntryCompletion()
+        global_search_completion = Gtk.EntryCompletion()
         global_search_completion.set_match_func(lambda *a: True)
-        global_search_completion.set_model(gtk.ListStore(
-                gtk.gdk.Pixbuf, str, str, int, str))
-        pixbuf_cell = gtk.CellRendererPixbuf()
-        global_search_completion.pack_start(pixbuf_cell, False)
+        global_search_completion.set_model(
+            Gtk.ListStore(GdkPixbuf.Pixbuf, str, str, int, str))
+        pixbuf_cell = Gtk.CellRendererPixbuf()
+        global_search_completion.pack_start(pixbuf_cell, expand=True)
         global_search_completion.add_attribute(pixbuf_cell, 'pixbuf', 0)
-        text_cell = gtk.CellRendererText()
-        global_search_completion.pack_start(text_cell)
+        text_cell = Gtk.CellRendererText()
+        global_search_completion.pack_start(text_cell, expand=True)
         global_search_completion.add_attribute(text_cell, "markup", 1)
         global_search_completion.props.popup_set_width = True
         self.global_search_entry.set_completion(global_search_completion)
@@ -367,7 +375,7 @@ class Main(Gtk.Application):
                     if icon:
                         text = common.to_xml(record_name)
                         pixbuf = common.IconFactory.get_pixbuf(
-                            icon, gtk.ICON_SIZE_BUTTON)
+                            icon, Gtk.IconSize.BUTTON)
                     else:
                         text = '<b>%s:</b>\n %s' % (
                             common.to_xml(model_name),
@@ -386,7 +394,7 @@ class Main(Gtk.Application):
 
         def changed(widget):
             search_text = widget.get_text()
-            gobject.timeout_add(300, update, widget, search_text)
+            GLib.timeout_add(300, update, widget, search_text)
 
         def activate(widget):
             def message():
@@ -409,9 +417,10 @@ class Main(Gtk.Application):
                 CONFIG['login.login'],
                 CONFIG['login.host'],
                 CONFIG['login.db'])
-        titles = [CONFIG['client.title']]
+        titles = []
         if value:
             titles.append(value)
+        titles.append(CONFIG['client.title'])
         self.header.set_title(' - '.join(titles))
         self.header.set_subtitle(login_info)
         try:
@@ -428,12 +437,12 @@ class Main(Gtk.Application):
 
     def favorite_set(self, *args):
         if self.menu_favorite.get_children():
-            return True
+            return
 
         def _action_favorite(widget, id_):
-            event = gtk.get_current_event()
-            allow_similar = (event.state & gtk.gdk.MOD1_MASK or
-                             event.state & gtk.gdk.SHIFT_MASK)
+            event = Gtk.get_current_event()
+            allow_similar = (event.state & Gdk.ModifierType.MOD1_MASK
+                or event.state & Gdk.ModifierType.SHIFT_MASK)
             with Window(allow_similar=allow_similar):
                 # ids is not defined to prevent to add suffix
                 Action.exec_keyword('tree_open', {
@@ -452,21 +461,15 @@ class Main(Gtk.Application):
         except Exception:
             return False
         for id_, name, icon in favorites:
-            if icon:
-                menuitem = gtk.ImageMenuItem(name)
-                menuitem.set_image(
-                    common.IconFactory.get_image(icon, gtk.ICON_SIZE_MENU))
-            else:
-                menuitem = gtk.MenuItem(name)
+            menuitem = Gtk.MenuItem(label=name)
             menuitem.connect('activate', _action_favorite, id_)
             self.menu_favorite.add(menuitem)
-        self.menu_favorite.add(gtk.SeparatorMenuItem())
-        manage_favorites = gtk.MenuItem(_("Manage..."),
+        self.menu_favorite.add(Gtk.SeparatorMenuItem())
+        manage_favorites = Gtk.MenuItem(label=_("Manage..."),
             use_underline=True)
         manage_favorites.connect('activate', _manage_favorites)
         self.menu_favorite.add(manage_favorites)
         self.menu_favorite.show_all()
-        return True
 
     def favorite_unset(self):
         for child in self.menu_favorite.get_children():
@@ -479,23 +482,20 @@ class Main(Gtk.Application):
         if option == 'default':
             barstyle = False
         elif option == 'both':
-            barstyle = gtk.TOOLBAR_BOTH
+            barstyle = Gtk.ToolbarStyle.BOTH
         elif option == 'text':
-            barstyle = gtk.TOOLBAR_TEXT
+            barstyle = Gtk.ToolbarStyle.TEXT
         elif option == 'icons':
-            barstyle = gtk.TOOLBAR_ICONS
+            barstyle = Gtk.ToolbarStyle.ICONS
         for page_idx in range(self.notebook.get_n_pages()):
             page = self.get_page(page_idx)
-            page.toolbar.set_style(barstyle)
+            if page.toolbar:
+                page.toolbar.set_style(barstyle)
 
     def edit_limit(self):
         from tryton.gui.window.limit import Limit
         Limit().run()
 
-    def edit_email(self):
-        from tryton.gui.window.email import Email
-        Email().run()
-
     def win_next(self):
         page = self.notebook.get_current_page()
         if page == len(self.pages) - 1:
@@ -507,45 +507,42 @@ class Main(Gtk.Application):
         self.notebook.set_current_page(page - 1)
 
     def get_preferences(self):
-        def _set_preferences(prefs):
-            try:
-                prefs = prefs()
-            except RPCException:
-                prefs = {}
-            threads = []
-            for target in (
-                    common.IconFactory.load_icons,
-                    common.MODELACCESS.load_models,
-                    common.MODELHISTORY.load_history,
-                    common.VIEW_SEARCH.load_searches,
-                    ):
-                t = threading.Thread(target=target)
-                threads.append(t)
-                t.start()
-            for t in threads:
-                t.join()
-            if prefs and 'language_direction' in prefs:
-                translate.set_language_direction(prefs['language_direction'])
-                CONFIG['client.language_direction'] = \
-                    prefs['language_direction']
-            self.sig_win_menu(prefs=prefs)
-            for action_id in prefs.get('actions', []):
-                Action.execute(action_id, {})
-            self.set_title(prefs.get('status_bar', ''))
-            if prefs and 'language' in prefs:
-                translate.setlang(prefs['language'], prefs.get('locale'))
-                if CONFIG['client.lang'] != prefs['language']:
-                    self.favorite_unset()
-                CONFIG['client.lang'] = prefs['language']
-            # Set placeholder after language is set to get correct translation
-            self.global_search_entry.set_placeholder_text(_("Action"))
-            CONFIG.save()
+        RPCContextReload()
+        try:
+            prefs = RPCExecute('model', 'res.user', 'get_preferences', False)
+        except RPCException:
+            prefs = {}
 
-        def _get_preferences():
-            RPCExecute('model', 'res.user', 'get_preferences', False,
-                callback=_set_preferences)
-
-        RPCContextReload(_get_preferences)
+        threads = []
+        for target in (
+                common.IconFactory.load_icons,
+                common.MODELACCESS.load_models,
+                common.MODELHISTORY.load_history,
+                common.VIEW_SEARCH.load_searches,
+                ):
+            t = threading.Thread(target=target)
+            threads.append(t)
+            t.start()
+        for t in threads:
+            t.join()
+        if prefs and 'language_direction' in prefs:
+            translate.set_language_direction(prefs['language_direction'])
+            CONFIG['client.language_direction'] = \
+                prefs['language_direction']
+        self.sig_win_menu(prefs=prefs)
+        for action_id in prefs.get('actions', []):
+            Action.execute(action_id, {})
+        self.set_title(prefs.get('status_bar', ''))
+        if prefs and 'language' in prefs:
+            translate.setlang(prefs['language'], prefs.get('locale'))
+            if CONFIG['client.lang'] != prefs['language']:
+                self.favorite_unset()
+                self.primary_menu.set_menu_model(self._get_primary_menu())
+            CONFIG['client.lang'] = prefs['language']
+        common.MODELNAME.clear()
+        # Set placeholder after language is set to get correct translation
+        self.global_search_entry.set_placeholder_text(_("Action"))
+        CONFIG.save()
 
     def preferences(self):
         from tryton.gui.window.preference import Preference
@@ -583,8 +580,114 @@ class Main(Gtk.Application):
         About()
 
     def shortcuts(self):
-        from tryton.gui.window.shortcuts import Shortcuts
-        Shortcuts().run()
+        if self._shortcuts:
+            self._shortcuts.destroy()
+        self._shortcuts = window = Gtk.ShortcutsWindow()
+        window.set_transient_for(common.get_toplevel_window())
+        window.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
+        window.set_destroy_with_parent(True)
+        self.add_window(window)
+
+        section = Gtk.ShortcutsSection()
+        section.props.section_name = 'app'
+        section.props.title = _("Application Shortcuts")
+        section.props.visible = True
+
+        group = Gtk.ShortcutsGroup()
+        group.props.title = _("Global")
+        section.add(group)
+
+        for action, title in [
+                ('app.preferences', _("Preferences")),
+                ('app.menu-search', _("Search menu")),
+                ('app.menu-toggle', _("Toggle menu")),
+                ('app.tab-previous', _("Previous tab"), ),
+                ('app.tab-next', _("Next tab")),
+                ('app.shortcuts', _("Shortcuts")),
+                ('app.quit', _("Quit")),
+                ]:
+            shortcut = Gtk.ShortcutsShortcut()
+            shortcut.props.title = title
+            accels = self.get_accels_for_action(action)
+            if accels:
+                shortcut.props.accelerator = ' '.join(accels)
+                group.add(shortcut)
+
+        window.add(section)
+
+        for name, title, groups in [
+                ('entry', _("Edition Shortcuts"), [
+                        (_("Text Entries"), [
+                                ('<Primary>x', _("Cut selected text")),
+                                ('<Primary>c', _("Copy selected text")),
+                                ('<Primary>v', _("Paste copied text")),
+                                ('Tab', _("Next entry")),
+                                ('<Shift>Tab', _("Previous entry")),
+                                ]),
+                        (_("Relation Entries"), [
+                                ('F3', _("Create new relation")),
+                                ('F2', _("Open/Search relation")),
+                                ]),
+                        (_("List Entries"), [
+                                ('F4', _("Switch view")),
+                                ('F3', _("Create/Select new line")),
+                                ('F2', _("Open relation")),
+                                ('Delete',
+                                    _("Mark line for deletion/removal")),
+                                ('<Ctrl>Delete', _("Mark line for removal")),
+                                ('Insert', _("Unmark line for deletion")),
+                                ]),
+                        ]),
+                ('tree', _("List/Tree Shortcuts"), [
+                        (_("Move Cursor"), [
+                                ('Right', _("Move right")),
+                                ('Left', _("Move left")),
+                                ('Up', _("Move up")),
+                                ('Down', _("Move down")),
+                                ('Page_Up', _("Move up of one page")),
+                                ('Page_Down', _("Move down of one page")),
+                                ('Home', _("Move to top")),
+                                ('End', _("Move to bottom")),
+                                ('BackSpace', _("Move to parent")),
+                                ]),
+                        (_("Selection"), [
+                                ('<Ctrl>a <Ctrl>slash', _("Select all")),
+                                ('<Shift><Ctrl>a <Shift><Ctrl>slash',
+                                    _("Unselect all")),
+                                ('BackSpace', _("Select parent")),
+                                ('space', _("Select/Activate current row")),
+                                ('<Shift>space Return',
+                                    _("Select/Activate current row")),
+                                ('<Ctrl>space', _("Toggle selection")),
+                                ]),
+                        (_("Expand/Collapse"), [
+                                ('plus', _("Expand row")),
+                                ('minus', _("Collapse row")),
+                                ('space', _("Toggle row")),
+                                ('<Shift>Left', _("Collapse all rows")),
+                                ('<Shift>Right', _("Expand all rows")),
+                                ]),
+                        ]),
+                ]:
+            section = Gtk.ShortcutsSection()
+            section.props.section_name = name
+            section.props.title = title
+            section.props.visible = True
+
+            for title, shortcuts in groups:
+                group = Gtk.ShortcutsGroup()
+                group.props.title = title
+                section.add(group)
+
+                for accelerator, title in shortcuts:
+                    shortcut = Gtk.ShortcutsShortcut()
+                    shortcut.props.title = title
+                    shortcut.props.accelerator = accelerator
+                    group.add(shortcut)
+
+            window.add(section)
+
+        window.show_all()
 
     def menu_toggle(self, *args):
         if self.menu.get_visible():
@@ -641,18 +744,19 @@ class Main(Gtk.Application):
         screen.switch_view(view_type=screen.current_view.view_type)
 
         self.menu.pack_start(
-            screen.screen_container.alternate_viewport, True, True)
+            screen.screen_container.alternate_viewport,
+            expand=True, fill=True, padding=0)
         treeview = screen.current_view.treeview
         treeview.set_headers_visible(False)
 
         # Favorite column
-        column = gtk.TreeViewColumn()
+        column = Gtk.TreeViewColumn()
         column.name = None
         column._type = None
         favorite_renderer = CellRendererClickablePixbuf()
         column.pack_start(favorite_renderer, expand=False)
 
-        def favorite_setter(column, cell, store, iter_):
+        def favorite_setter(column, cell, store, iter_, user_data=None):
             menu = store.get_value(iter_, 0)
             favorite = menu.value.get('favorite')
             if favorite:
@@ -663,7 +767,7 @@ class Main(Gtk.Application):
                 icon = None
             if icon:
                 pixbuf = common.IconFactory.get_pixbuf(
-                    icon, gtk.ICON_SIZE_MENU)
+                    icon, Gtk.IconSize.MENU)
             else:
                 pixbuf = None
             cell.set_property('pixbuf', pixbuf)
@@ -673,11 +777,11 @@ class Main(Gtk.Application):
             if treeview.props.window:
                 self.toggle_favorite(renderer, path, treeview)
         favorite_renderer.connect('clicked',
-            lambda *a: gobject.idle_add(toggle_favorite, *a), treeview)
+            lambda *a: GLib.idle_add(toggle_favorite, *a), treeview)
         # Unset fixed height mode to add column
         treeview.set_fixed_height_mode(False)
         treeview.set_property(
-            'enable-grid-lines', gtk.TREE_VIEW_GRID_LINES_NONE)
+            'enable-grid-lines', Gtk.TreeViewGridLines.NONE)
         treeview.append_column(column)
 
         screen.search_filter()
@@ -724,34 +828,35 @@ class Main(Gtk.Application):
             page_id = -1
         self.previous_pages[page] = previous_widget
         self.pages.append(page)
-        hbox = gtk.HBox(spacing=3)
+        hbox = Gtk.HBox(spacing=3)
         if page.icon:
             hbox.pack_start(
                 common.IconFactory.get_image(
-                    page.icon, gtk.ICON_SIZE_SMALL_TOOLBAR),
-                expand=False, fill=False)
+                    page.icon, Gtk.IconSize.SMALL_TOOLBAR),
+                expand=False, fill=False, padding=0)
         name = page.name
-        label = gtk.Label(common.ellipsize(name, 20))
+        label = Gtk.Label(
+            label=common.ellipsize(name, 20),
+            halign=Gtk.Align.START)
         self.tooltips.set_tip(label, page.name)
         self.tooltips.enable()
-        label.set_alignment(0.0, 0.5)
-        hbox.pack_start(label, expand=True, fill=True)
+        hbox.pack_start(label, expand=True, fill=True, padding=0)
 
-        button = gtk.Button()
-        button.set_relief(gtk.RELIEF_NONE)
+        button = Gtk.Button()
+        button.set_relief(Gtk.ReliefStyle.NONE)
         button.set_can_focus(False)
         button.add(common.IconFactory.get_image(
-                'tryton-close', gtk.ICON_SIZE_MENU))
+                'tryton-close', Gtk.IconSize.MENU))
         self.tooltips.set_tip(button, _('Close Tab'))
         button.connect('clicked', self._sig_remove_book, page.widget)
-        hbox.pack_start(button, expand=False, fill=False)
-        x, y = gtk.icon_size_lookup_for_settings(button.get_settings(),
-            gtk.ICON_SIZE_MENU)[-2:]
-        button.set_size_request(x, y)
+        hbox.pack_start(button, expand=False, fill=False, padding=0)
+        size = Gtk.IconSize.lookup(Gtk.IconSize.MENU)
+        button.set_size_request(size.width, size.height)
 
         hbox.show_all()
-        label_menu = gtk.Label(page.name)
-        label_menu.set_alignment(0.0, 0.5)
+        label_menu = Gtk.Label(
+            label=page.name, halign=Gtk.Align.START)
+        page.widget.props.margin = 3
         self.notebook.insert_page_menu(page.widget, hbox, label_menu, page_id)
         self.notebook.set_tab_reorderable(page.widget, True)
         self.notebook.set_current_page(page_id)
@@ -833,9 +938,9 @@ class Main(Gtk.Application):
         def set_cursor():
             if self.current_page == self.notebook.get_current_page():
                 current_form.set_cursor()
-        # Using idle_add because the gtk.TreeView grabs the focus at the
+        # Using idle_add because the Gtk.TreeView grabs the focus at the
         # end of the event
-        gobject.idle_add(set_cursor)
+        GLib.idle_add(set_cursor)
         for dialog in current_form.dialogs:
             dialog.show()
 
@@ -874,6 +979,8 @@ class Main(Gtk.Application):
                 context = json.loads(params.get('context', '{}'),
                     object_hook=object_hook)
                 context_model = params.get('context_model')
+                tab_domain = json.loads(params.get('tab_domain', '[]'),
+                    object_hook=object_hook)
             except ValueError:
                 return
             if path:
@@ -892,7 +999,8 @@ class Main(Gtk.Application):
                     mode=mode,
                     name=name,
                     limit=limit,
-                    search_value=search_value)
+                    search_value=search_value,
+                    tab_domain=tab_domain)
             except Exception:
                 # Prevent crashing the client
                 return
@@ -904,8 +1012,6 @@ class Main(Gtk.Application):
                 data = json.loads(params.get('data', '{}'),
                     object_hook=object_hook)
                 direct_print = json.loads(params.get('direct_print', 'false'))
-                email_print = json.loads(params.get('email_print', 'false'))
-                email = json.loads(params.get('email', 'null'))
                 name = json.loads(params.get('name', '""'))
                 window = json.loads(params.get('window', 'false'))
                 context = json.loads(params.get('context', '{}'),
@@ -913,8 +1019,8 @@ class Main(Gtk.Application):
             except ValueError:
                 return
             try:
-                Window.create_wizard(wizard, data, direct_print=direct_print,
-                    email_print=email_print, email=email, name=name,
+                Window.create_wizard(
+                    wizard, data, direct_print=direct_print, name=name,
                     context=context, window=window)
             except Exception:
                 # Prevent crashing the client
@@ -926,15 +1032,13 @@ class Main(Gtk.Application):
             try:
                 data = json.loads(params.get('data'), object_hook=object_hook)
                 direct_print = json.loads(params.get('direct_print', 'false'))
-                email_print = json.loads(params.get('email_print', 'false'))
-                email = json.loads(params.get('email', 'null'))
                 context = json.loads(params.get('context', '{}'),
                     object_hook=object_hook)
             except ValueError:
                 return
             try:
-                Action.exec_report(report, data, direct_print=direct_print,
-                    email_print=email_print, email=email, context=context)
+                Action.exec_report(
+                    report, data, direct_print=direct_print, context=context)
             except Exception:
                 # Prevent crashing the client
                 return
@@ -960,10 +1064,9 @@ class Main(Gtk.Application):
 
     def open_url(self, url):
         def idle_open_url():
-            with gtk.gdk.lock:
-                self._open_url(url)
-                return False
-        gobject.idle_add(idle_open_url)
+            self._open_url(url)
+            return False
+        GLib.idle_add(idle_open_url)
 
     def show_notification(self, title, msg, priority=1):
         notification = Gio.Notification.new(title)
