@@ -32,5 +32,16 @@ class NoModal(object):
         self.sensible_widget.props.sensitive = True
         for focus in self.parent_focus:
             if focus and focus.is_ancestor(self.parent):
-                focus.grab_focus()
+                try:
+                    focus.grab_focus()
+                except TypeError:
+                    # GooCanvas needs a GooCanvasItem
+                    continue
                 break
+
+    def default_size(self):
+        from tryton.gui.main import Main
+        main = Main()
+        allocation = main.window.get_allocation()
+        width, height = allocation.width, allocation.height
+        return max(width - 150, 0), max(height - 150, 0)
