@@ -28,9 +28,11 @@ class Record(SignalEvent):
         self.state_attrs = {}
         self.modified_fields = {}
         self._timestamp = None
-        self.attachment_count = -1
-        self.unread_note = -1
+        self._write = True
+        self._delete = True
+        self.resources = None
         self.button_clicks = {}
+        self.links_counts = {}
         self.next = {}  # Used in Group list
         self.value = {}
         self.autocompletion = {}
@@ -73,7 +75,7 @@ class Record(SignalEvent):
                     in ('many2one', 'one2one', 'reference')))
             if 'rec_name' not in fnames:
                 fnames.append('rec_name')
-            fnames.append('_timestamp')
+            fnames.extend(['_timestamp', '_write', '_delete'])
 
             record_context = self.get_context()
             if loading == 'eager':
@@ -136,7 +138,8 @@ class Record(SignalEvent):
                     for key in record.modified_fields:
                         value.pop(key, None)
                     record.set(value, signal=False)
-        return self.group.fields[name]
+        if name != '*':
+            return self.group.fields[name]
 
     def __repr__(self):
         return '<Record %s@%s at %s>' % (self.id, self.model_name, id(self))
@@ -228,10 +231,17 @@ class Record(SignalEvent):
     deleted = property(get_deleted)
 
     def get_readonly(self):
-        return self.deleted or self.removed or self.exception
+        return (self.deleted
+            or self.removed
+            or self.exception
+            or not self._write)
 
     readonly = property(get_readonly)
 
+    @property
+    def deletable(self):
+        return self._delete
+
     def fields_get(self):
         return self.group.fields
 
@@ -257,7 +267,8 @@ class Record(SignalEvent):
         value = {}
         for name, field in self.group.fields.items():
             if (field.attrs.get('readonly')
-                    and not isinstance(field, fields.O2MField)):
+                    and not (isinstance(field, fields.O2MField)
+                        and not isinstance(field, fields.M2MField))):
                 continue
             if field.name not in self.modified_fields and self.id >= 0:
                 continue
@@ -294,6 +305,7 @@ class Record(SignalEvent):
         self.modified_fields.clear()
         self._timestamp = None
         self.button_clicks.clear()
+        self.links_counts.clear()
 
     def get_timestamp(self):
         result = {self.model_name + ',' + str(self.id): self._timestamp}
@@ -305,7 +317,7 @@ class Record(SignalEvent):
     def pre_validate(self):
         if not self.modified_fields:
             return True
-        values = self._get_on_change_args(self.modified_fields)
+        values = self._get_on_change_args(['id'] + list(self.modified_fields))
         try:
             RPCExecute('model', self.model_name, 'pre_validate', values,
                 context=self.get_context())
@@ -315,8 +327,8 @@ class Record(SignalEvent):
 
     def save(self, force_reload=True):
         if self.id < 0 or self.modified:
+            value = self.get()
             if self.id < 0:
-                value = self.get()
                 try:
                     res, = RPCExecute('model', self.model_name, 'create',
                         [value], context=self.get_context())
@@ -326,7 +338,6 @@ class Record(SignalEvent):
                 self.id = res
                 self.group.id_changed(old_id)
             elif self.modified:
-                value = self.get()
                 if value:
                     context = self.get_context()
                     context = context.copy()
@@ -380,7 +391,7 @@ class Record(SignalEvent):
         elif fields is None:
             self._check_load()
         res = True
-        for field_name, field in self.group.fields.items():
+        for field_name, field in list(self.group.fields.items()):
             if fields is not None and field_name not in fields:
                 continue
             if field.attrs.get('readonly'):
@@ -416,11 +427,8 @@ class Record(SignalEvent):
                 continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in val:
-                    self.value[field_rec_name] = val[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                related = fieldname + '.'
+                self.value[related] = val.get(related) or {}
             self.group.fields[fieldname].set_default(self, value)
             self._loaded.add(fieldname)
             fieldnames.append(fieldname)
@@ -440,6 +448,9 @@ class Record(SignalEvent):
                 if not self._timestamp:
                     self._timestamp = value
                 continue
+            if fieldname in {'_write', '_delete'}:
+                setattr(self, fieldname, value)
+                continue
             if fieldname not in self.group.fields:
                 if fieldname == 'rec_name':
                     self.value['rec_name'] = value
@@ -449,11 +460,8 @@ class Record(SignalEvent):
                 continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in val:
-                    self.value[field_rec_name] = val[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                related = fieldname + '.'
+                self.value[related] = val.get(related) or {}
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
             fieldnames.append(fieldname)
@@ -471,12 +479,10 @@ class Record(SignalEvent):
                 continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in values:
-                    self.value[field_rec_name] = values[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
-            self.group.fields[fieldname].set_on_change(self, value)
+                related = fieldname + '.'
+                self.value[related] = values.get(related) or {}
+            # Load fieldname before setting value
+            self[fieldname].set_on_change(self, value)
 
     def reload(self, fields=None):
         if self.id < 0:
@@ -528,7 +534,6 @@ class Record(SignalEvent):
                 scope = scope[i]
             else:
                 res[arg] = scope
-        res['id'] = self.id
         return res
 
     def on_change(self, fieldnames):
@@ -541,8 +546,16 @@ class Record(SignalEvent):
 
         if values:
             try:
-                changes = RPCExecute('model', self.model_name, 'on_change',
-                    values, fieldnames, context=self.get_context())
+                if len(fieldnames) == 1:
+                    fieldname, = fieldnames
+                    changes = []
+                    changes.append(RPCExecute(
+                            'model', self.model_name, 'on_change_' + fieldname,
+                            values, context=self.get_context()))
+                else:
+                    changes = RPCExecute(
+                        'model', self.model_name, 'on_change',
+                        values, fieldnames, context=self.get_context())
             except RPCException:
                 return
             for change in changes:
@@ -567,13 +580,20 @@ class Record(SignalEvent):
             values.update(self._get_on_change_args(on_change_with))
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                self.value.pop(fieldname + '.', None)
         if fieldnames:
             try:
-                result = RPCExecute('model', self.model_name, 'on_change_with',
-                    values, list(fieldnames), context=self.get_context())
+                if len(fieldnames) == 1:
+                    fieldname, = fieldnames
+                    result = {}
+                    result[fieldname] = RPCExecute(
+                        'model', self.model_name,
+                        'on_change_with_' + fieldname,
+                        values, context=self.get_context())
+                else:
+                    result = RPCExecute(
+                        'model', self.model_name, 'on_change_with',
+                        values, list(fieldnames), context=self.get_context())
             except RPCException:
                 return
             self.set_on_change(result)
@@ -587,7 +607,8 @@ class Record(SignalEvent):
                     context=self.get_context())
             except RPCException:
                 return
-            self.group.fields[fieldname].set_on_change(self, result)
+            # Load fieldname before setting value
+            self[fieldname].set_on_change(self, result)
 
     def autocomplete_with(self, field_name):
         for fieldname, fieldinfo in self.group.fields.items():
@@ -601,8 +622,9 @@ class Record(SignalEvent):
         autocomplete = self.group.fields[fieldname].attrs['autocomplete']
         args = self._get_on_change_args(autocomplete)
         try:
-            res = RPCExecute('model', self.model_name, 'autocomplete_' +
-                fieldname, args, context=self.get_context())
+            res = RPCExecute(
+                'model', self.model_name,
+                'autocomplete_' + fieldname, args, context=self.get_context())
         except RPCException:
             # ensure res is a list
             res = []
@@ -618,34 +640,15 @@ class Record(SignalEvent):
             if context:
                 value.context = self.expr_eval(context)
 
-    def get_attachment_count(self, reload=False):
-        if self.id < 0:
-            return 0
-        if self.attachment_count < 0 or reload:
+    def get_resources(self, reload=False):
+        if self.id >= 0 and (not self.resources or reload):
             try:
-                self.attachment_count = RPCExecute('model', 'ir.attachment',
-                    'search_count', [
-                        ('resource', '=',
-                            '%s,%s' % (self.model_name, self.id)),
-                        ], context=self.get_context())
+                self.resources = RPCExecute(
+                    'model', self.model_name, 'resources', self.id,
+                    context=self.get_context())
             except RPCException:
-                return 0
-        return self.attachment_count
-
-    def get_unread_note(self, reload=False):
-        if self.id < 0:
-            return 0
-        if self.unread_note < 0 or reload:
-            try:
-                self.unread_note = RPCExecute('model', 'ir.note',
-                    'search_count', [
-                        ('resource', '=',
-                            '%s,%s' % (self.model_name, self.id)),
-                        ('unread', '=', True),
-                        ], context=self.get_context())
-            except RPCException:
-                return 0
-        return self.unread_note
+                pass
+        return self.resources
 
     def get_button_clicks(self, name):
         if self.id < 0:
